#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# SAT: a jabber client
# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" XMPP uri parsing tools """

import urlparse
import urllib

# FIXME: basic implementation, need to follow RFC 5122

def parseXMPPUri(uri):
    """Parse an XMPP uri and return a dict with various information

    @param uri(unicode): uri to parse
    @return dict(unicode, unicode): data depending of the URI where key can be:
        type: one of ("pubsub", TODO)
        sub_type: can be:
            - microblog
            only used for pubsub for now
        path: XMPP path (jid of the service or entity)
        node: node used
        id: id of the element (item for pubsub)
    @raise ValueError: the scheme is not xmpp
    """
    uri_split = urlparse.urlsplit(uri.encode('utf-8'))
    if uri_split.scheme != 'xmpp':
        raise ValueError(u'this is not a XMPP URI')

    data = {u'path': urllib.unquote(uri_split.path).decode('utf-8')}

    query_end = uri_split.query.find(';')
    query_type = uri_split.query[:query_end]
    if query_end == -1 or '=' in query_type:
        raise ValueError('no query type, invalid XMPP URI')

    pairs = urlparse.parse_qs(uri_split.geturl())
    for k, v in pairs.items():
        if len(v) != 1:
            raise NotImplementedError(u"multiple values not managed")
        if k in ('path', 'type', 'sub_type'):
            raise NotImplementedError(u"reserved key used in URI, this is not supported")
        data[k.decode('utf-8')] = urllib.unquote(v[0]).decode('utf-8')

    if query_type:
        data[u'type'] = query_type.decode('utf-8')
    elif u'node' in data:
        data[u'type'] = u'pubsub'
    else:
        data[u'type'] = ''

    if u'node' in data:
        if data[u'node'].startswith(u'urn:xmpp:microblog:'):
            data[u'sub_type'] = 'microblog'

    return data
